@@ -1,6 +1,7 @@
 #
 #  Copyright 2015, 2016 Hewlett Packard Enterprise Development Company LP
 #  Copyright 2016 Cray Inc.
+#  Copyright 2017 Intel Corporation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,10 +25,9 @@ from horizon.utils.memoized import memoized  # noqa
 from openstack_dashboard.api import base
 
 
-DEFAULT_IRONIC_API_VERSION = '1.20'
+DEFAULT_IRONIC_API_VERSION = '1.31'
 DEFAULT_INSECURE = False
 DEFAULT_CACERT = None
-
 IRONIC_CLIENT_CLASS_NAME = 'baremetal'
 
 
@@ -87,17 +87,20 @@ def node_list_ports(request, node_id):
     return ironicclient(request).node.list_ports(node_id, limit=0, detail=True)
 
 
-def node_set_power_state(request, node_id, state):
+def node_set_power_state(request, node_id, state, soft=False):
     """Set power state for a given node.
 
     :param request: HTTP request.
     :param node_id: The UUID or name of the node.
-    :param state: the power state to set.
+    :param state: the power state to set ['on', 'off', 'reboot'].
+    :param soft: flag for graceful power 'off' or reboot
     :return: node.
 
     http://docs.openstack.org/developer/python-ironicclient/api/ironicclient.v1.node.html#ironicclient.v1.node.NodeManager.set_power_state
     """
-    return ironicclient(request).node.set_power_state(node_id, state)
+    return ironicclient(request).node.set_power_state(node_id,
+                                                      state,
+                                                      soft)
 
 
 def node_set_provision_state(request, node_id, state, cleansteps=None):
@@ -117,6 +120,31 @@ def node_set_provision_state(request, node_id, state, cleansteps=None):
                                             cleansteps=cleansteps)
 
 
+def node_set_console_mode(request, node_id, enabled):
+    """Start or stop the serial console for a given node.
+
+    :param request: HTTP request.
+    :param node_id: The UUID or name of the node.
+    :param enabled: True to start the console, False to stop it
+    :return: node.
+
+    http://docs.openstack.org/developer/python-ironicclient/api/ironicclient.v1.node.html#ironicclient.v1.node.NodeManager.set_console_mode
+    """
+    return ironicclient(request).node.set_console_mode(node_id, enabled)
+
+
+def node_get_console(request, node_id):
+    """Get connection information for a node's console.
+
+    :param request: HTTP request.
+    :param node_id: The UUID or name of the node.
+    :return: Console connection information
+
+    http://docs.openstack.org/developer/python-ironicclient/api/ironicclient.v1.node.html#ironicclient.v1.node.NodeManager.get_console
+    """
+    return ironicclient(request).node.get_console(node_id)
+
+
 def node_set_maintenance(request, node_id, state, maint_reason=None):
     """Set the maintenance mode on a given node.
 
@@ -190,6 +218,47 @@ def node_validate(request, node_id):
     return result
 
 
+def node_get_boot_device(request, node_id):
+    """Get the boot device for a specified node.
+
+    :param request: HTTP request.
+    :param node_id: The UUID or name of the node.
+    :return: Dictionary with keys "boot_device" and "persistent"
+
+    http://docs.openstack.org/developer/python-ironicclient/api/ironicclient.v1.node.html#ironicclient.v1.node.NodeManager.get_boot_device
+    """
+    return ironicclient(request).node.get_boot_device(node_id)
+
+
+def node_set_boot_device(request, node_id, device, persistent):
+    """Set the boot device for a specified node.
+
+    :param request: HTTP request.
+    :param node_id: The UUID or name of the node.
+    :param device: boot device.
+    :param persistent: True or False.
+    :return: null.
+
+    http://docs.openstack.org/developer/python-ironicclient/api/ironicclient.v1.node.html#ironicclient.v1.node.NodeManager.set_boot_device
+    """
+    return ironicclient(request).node.set_boot_device(node_id,
+                                                      device,
+                                                      persistent)
+
+
+def node_get_supported_boot_devices(request, node_id):
+    """Get the list of supported boot devices for a specified node.
+
+    :param request: HTTP request.
+    :param node_id: The UUID or name of the node.
+    :return: List of supported boot devices (strings)
+
+    http://docs.openstack.org/developer/python-ironicclient/api/ironicclient.v1.node.html#ironicclient.v1.node.NodeManager.get_boot_device
+    """
+    result = ironicclient(request).node.get_supported_boot_devices(node_id)
+    return result.get('supported_boot_devices', [])
+
+
 def driver_list(request):
     """Retrieve a list of drivers.
 
@@ -238,16 +307,80 @@ def port_delete(request, port_uuid):
     return ironicclient(request).port.delete(port_uuid)
 
 
-def port_update(request, port_id, patch):
+def port_update(request, port_uuid, patch):
     """Update a specified port.
 
     :param request: HTTP request.
-    :param node_id: The uuid of the port.
+    :param port_id: The UUID of the port.
     :param patch: Sequence of update operations
-    :return: port.
+    :return: Port.
 
     http://docs.openstack.org/developer/python-ironicclient/api/ironicclient.v1.port.html#ironicclient.v1.port.PortManager.update
     """
-    port = ironicclient(request).port.update(port_id, patch)
+    port = ironicclient(request).port.update(port_uuid, patch)
     return dict([(f, getattr(port, f, ''))
                  for f in res_fields.PORT_DETAILED_RESOURCE.fields])
+
+
+def portgroup_list(request, node_id):
+    """List the portgroups associated with a given node.
+
+    :param request: HTTP request.
+    :param node_id: The UUID or name of the node.
+    :return: A full list of portgroups. (limit=0)
+
+    http://docs.openstack.org/developer/python-ironicclient/api/ironicclient.v1.portgroup.html#ironicclient.v1.portgroup.PortgroupManager.list_portgroups
+    """
+    return ironicclient(request).portgroup.list(node_id, limit=0, detail=True)
+
+
+def portgroup_create(request, params):
+    """Create a portgroup.
+
+    :param request: HTTP request.
+    :param params: Portgroup creation parameters.
+    :return: Portgroup.
+
+    http://docs.openstack.org/developer/python-ironicclient/api/ironicclient.v1.portgroup.html#ironicclient.v1.portgroup.PortgroupManager.create
+    """
+    portgroup_manager = ironicclient(request).portgroup
+    return portgroup_manager.create(**params)
+
+
+def portgroup_delete(request, portgroup_id):
+    """Delete a portgroup from the DB.
+
+    :param request: HTTP request.
+    :param portgroup_id: The UUID or name of the portgroup.
+    :return: Portgroup.
+
+    http://docs.openstack.org/developer/python-ironicclient/api/ironicclient.v1.portgroup.html#ironicclient.v1.portgroup.PortgroupManager.delete
+    """
+    return ironicclient(request).portgroup.delete(portgroup_id)
+
+
+def portgroup_update(request, portgroup_id, patch):
+    """Update a specified portgroup.
+
+    :param request: HTTP request.
+    :param portgroup_id: The UUID or name of the portgroup.
+    :param patch: Sequence of update operations
+    :return: Portgroup.
+
+    http://docs.openstack.org/developer/python-ironicclient/api/ironicclient.v1.port.html#ironicclient.v1.portgroup.PortgroupManager.update
+    """
+    portgroup = ironicclient(request).portgroup.update(portgroup_id, patch)
+    return dict([(f, getattr(portgroup, f, ''))
+                 for f in res_fields.PORTGROUP_DETAILED_RESOURCE.fields])
+
+
+def portgroup_get_ports(request, portgroup_id):
+    """Get the ports associated with a specified portgroup.
+
+    :param request: HTTP request.
+    :param portgroup_id: The UUID or name of the portgroup.
+    :return: List of ports.
+
+    http://docs.openstack.org/developer/python-ironicclient/api/ironicclient.v1.portgroup.html#ironicclient.v1.portgroup.PortgroupManager.list_ports
+    """
+    return ironicclient(request).portgroup.list_ports(portgroup_id)
