@@ -1,6 +1,7 @@
 /*
  * © Copyright 2015,2016 Hewlett Packard Enterprise Development Company LP
  * © Copyright 2016 Cray Inc.
+ * Copyright 2017 Intel Corporation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -49,14 +50,22 @@
       getNode: getNode,
       getNodes: getNodes,
       getPortsWithNode: getPortsWithNode,
-      powerOffNode: powerOffNode,
-      powerOnNode: powerOnNode,
-      putNodeInMaintenanceMode: putNodeInMaintenanceMode,
-      removeNodeFromMaintenanceMode: removeNodeFromMaintenanceMode,
+      getBootDevice: getBootDevice,
+      getSupportedBootDevices: getSupportedBootDevices,
+      nodeGetConsole: nodeGetConsole,
+      nodeSetConsoleMode: nodeSetConsoleMode,
+      nodeSetMaintenance: nodeSetMaintenance,
+      nodeSetBootDevice: nodeSetBootDevice,
+      nodeSetPowerState: nodeSetPowerState,
       setNodeProvisionState: setNodeProvisionState,
       updateNode: updateNode,
       updatePort: updatePort,
-      validateNode: validateNode
+      updatePortgroup: updatePortgroup,
+      validateNode: validateNode,
+      createPortgroup: createPortgroup,
+      getPortgroups: getPortgroups,
+      deletePortgroup: deletePortgroup,
+      getPortgroupPorts: getPortgroupPorts
     };
 
     return service;
@@ -111,6 +120,52 @@
         });
     }
 
+    /**
+     * @description Retrieve the boot device for a node
+     * https://developer.openstack.org/api-ref/baremetal/#get-boot-device
+     *
+     * @param {string} nodeId – UUID or logical name of a node.
+     * @return {promise} Dictionary describing the current boot device
+     */
+    function getBootDevice(nodeId) {
+      return apiService.get('/api/ironic/nodes/' + nodeId + '/boot_device')
+        .then(function(response) {
+          return response.data;
+        })
+        .catch(function(response) {
+          var msg = interpolate(
+            gettext('Unable to retrieve boot device for Ironic node. %s'),
+            [response.data],
+            false);
+          toastService.add('error', msg);
+          return $q.reject(msg);
+        });
+    }
+
+    /**
+     * @description Retrieve the supported boot devices for a node
+     * https://developer.openstack.org/api-ref/baremetal/#get-supported-boot-devices
+     *
+     * @param {string} nodeId – UUID or logical name of a node.
+     * @return {promise} List of supported boot devices
+     */
+    function getSupportedBootDevices(nodeId) {
+      return apiService.get('/api/ironic/nodes/' + nodeId +
+                            '/boot_device/supported')
+        .then(function(response) {
+          return response.data; // List of supported boot devices
+        })
+        .catch(function(response) {
+          var msg = interpolate(
+            gettext(
+              'Unable to retrieve supported boot devices for Ironic node. %s'),
+            [response.data],
+            false);
+          toastService.add('error', msg);
+          return $q.reject(msg);
+        });
+    }
+
     /**
      * @description Retrieve a list of ports associated with a node.
      *
@@ -146,71 +201,54 @@
     }
 
     /**
-     * @description Put the node in maintenance mode.
+     * @description Set the maintenance state of a node
      *
      * http://developer.openstack.org/api-ref/baremetal/#set-maintenance-flag
      *
-     * @param {string} uuid – UUID or logical name of a node.
-     * @param {string} reason – Reason for why node is being put into
-     * maintenance mode
+     * @param {string} nodeId – UUID or logical name of a node.
+     * @param {boolean} mode - True to put the node in maintenance mode,
+     *   false to remove it from maintenance mode.
+     * @param {string} reason - Reason for putting the node in maintenance.
      * @return {promise} Promise
      */
-    function putNodeInMaintenanceMode(uuid, reason) {
-      return apiService.patch('/api/ironic/nodes/' + uuid + '/maintenance',
-                              {maint_reason: reason
-                               ? reason
-                               : gettext("No reason given.")})
-        .catch(function(response) {
-          var msg = interpolate(
-            gettext('Unable to put the Ironic node %s in maintenance mode: %s'),
-            [uuid, response.data],
-            false);
-          toastService.add('error', msg);
-          return $q.reject(msg);
-        });
+    function nodeSetMaintenance(nodeId, mode, reason) {
+      var url = '/api/ironic/nodes/' + nodeId + '/maintenance';
+      var promise = mode
+        ? apiService.patch(url,
+                           {maint_reason: reason ? reason
+                            : gettext("No reason given.")})
+        : apiService.delete(url);
+
+      return promise.catch(function(response) {
+        var msg = interpolate(
+          gettext('Unable to set Ironic node %s maintenance state: %s'),
+          [nodeId, response.data],
+          false);
+        toastService.add('error', msg);
+        return $q.reject(msg);
+      });
     }
 
     /**
-     * @description Remove the node from maintenance mode.
+     * @description Set the boot device of a node
      *
-     * http://developer.openstack.org/api-ref/baremetal/#clear-maintenance-flag
+     * http://developer.openstack.org/api-ref/baremetal/#set-boot-device
      *
-     * @param {string} uuid – UUID or logical name of a node.
+     * @param {string} nodeId – UUID or logical name of a node.
+     * @param {string} bootDevice - Selected boot device.
+     * @param {Boolean} persistent - True or False.
      * @return {promise} Promise
      */
-    function removeNodeFromMaintenanceMode(uuid) {
-      return apiService.delete('/api/ironic/nodes/' + uuid + '/maintenance')
-        .catch(function(response) {
-          var msg = interpolate(
-            gettext(
-              'Unable to remove the Ironic node %s from maintenance mode: %s'),
-            [uuid, response.data],
-            false);
-          toastService.add('error', msg);
-          return $q.reject(msg);
-        });
-    }
-
-    /**
-     * @description Set the power state of the node.
-     *
-     * http://developer.openstack.org/api-ref/baremetal/#change-node-power-state
-     *
-     * @param {string} uuid – UUID or logical name of a node.
-     * @return {promise} Promise
-     */
-    function powerOnNode(uuid) {
-      var data = {
-        state: 'on'
-      };
-      return apiService.patch('/api/ironic/nodes/' + uuid + '/states/power',
-                              data)
+    function nodeSetBootDevice(nodeId, bootDevice, persistent) {
+      return apiService.put('/api/ironic/nodes/' + nodeId + '/boot_device',
+                            {boot_device: bootDevice,
+                             persistent: persistent})
         .then(function() {
           toastService.add('success',
-                           gettext('Refresh page to see updated power status'));
+                           gettext('Refresh page to see set boot device'));
         })
         .catch(function(response) {
-          var msg = interpolate(gettext('Unable to power on the node: %s'),
+          var msg = interpolate(gettext('Unable to set boot device: %s'),
                                 [response.data],
                                 false);
           toastService.add('error', msg);
@@ -224,12 +262,17 @@
      * http://developer.openstack.org/api-ref/baremetal/#change-node-power-state
      *
      * @param {string} uuid – UUID or logical name of a node.
+     * @param {string} state - Target power state ['on', 'off', 'reboot']
+     * @param {boolean} soft - Flag for graceful power 'off' or reboot
      * @return {promise} Promise
      */
-    function powerOffNode(uuid) {
+    function nodeSetPowerState(uuid, state, soft) {
       var data = {
-        state: 'off'
+        state: state
       };
+      if (angular.isDefined(soft)) {
+        data.soft = soft;
+      }
       return apiService.patch('/api/ironic/nodes/' + uuid + '/states/power',
                               data)
         .then(function() {
@@ -491,6 +534,167 @@
           return $q.reject(msg);
         });
     }
-  }
 
+    /**
+     * @description Set the console mode of the node.
+     *
+     * http://developer.openstack.org/api-ref/baremetal/?
+     * expanded=start-stop-console-detail#start-stop-console
+     *
+     * @param {string} uuid – UUID of a node.
+     * @param {boolean} enabled – true to start the console, false to stop it
+     * @return {promise} Promise
+     */
+    function nodeSetConsoleMode(uuid, enabled) {
+      return apiService.put('/api/ironic/nodes/' + uuid + '/states/console',
+                            {enabled: enabled})
+        .then(function(response) {
+          var msg = gettext('Refresh page to see updated console details');
+          toastService.add('success', interpolate(msg, [uuid], false));
+          return response.data;
+        })
+        .catch(function(response) {
+          var msg = gettext('Unable to set console mode: %s');
+          toastService.add('error', interpolate(msg, [response.data], false));
+          return $q.reject(msg);
+        });
+    }
+
+    function nodeGetConsole(uuid) {
+      return apiService.get('/api/ironic/nodes/' + uuid + '/states/console')
+        .then(function(response) {
+          return response.data; // Object containing console information
+        })
+        .catch(function(response) {
+          var msg = gettext('Unable to get console for node %s: %s');
+          toastService.add('error',
+                           interpolate(msg, [uuid, response.data], false));
+          return $q.reject(msg);
+        });
+    }
+
+    /**
+     * @description Retrieve a list of portgroups associated with a node.
+     *
+     * http://developer.openstack.org/api-ref/baremetal/#list-detailed-portgroups
+     *
+     * @param {string} nodeId – UUID or logical name of a node.
+     * @return {promise} List of portgroups.
+     */
+    function getPortgroups(nodeId) {
+      return apiService.get('/api/ironic/portgroups/',
+                            {params: {node_id: nodeId}})
+        .then(function(response) {
+          // Add id property to support delete operations
+          // using the deleteModalService
+          angular.forEach(response.data.portgroups, function(portgroup) {
+            portgroup.id = portgroup.uuid;
+          });
+          return response.data.portgroups;
+        })
+        .catch(function(response) {
+          var msg = interpolate(
+            gettext('Unable to retrieve Ironic node portgroups: %s'),
+            [response.data],
+            false);
+          toastService.add('error', msg);
+          return $q.reject(msg);
+        });
+    }
+
+    /**
+     * @description Create a protgroup.
+     *
+     * http://developer.openstack.org/api-ref/baremetal/#create-portgroup
+     *
+     * @param {object} params – Object containing parameters that define
+     *   the portgroup to be created.
+     * @return {promise} Promise containing the portgroup.
+     */
+    function createPortgroup(params) {
+      return apiService.post('/api/ironic/portgroups/', params)
+        .then(function(response) {
+          toastService.add('success',
+                           gettext('Portgroup successfully created'));
+          return response.data; // The newly created portgroup
+        })
+        .catch(function(response) {
+          var msg = interpolate(gettext('Unable to create portgroup: %s'),
+                                [response.data],
+                                false);
+          toastService.add('error', msg);
+          return $q.reject(msg);
+        });
+    }
+
+    /**
+     * @description Delete a portgroup.
+     *
+     * http://developer.openstack.org/api-ref/baremetal/#delete-portgroup
+     *
+     * @param {string} portgroupId – UUID or name of the portgroup to be deleted.
+     * @return {promise} Promise.
+     */
+    function deletePortgroup(portgroupId) {
+      return apiService.delete('/api/ironic/portgroups/',
+                               {portgroup_id: portgroupId})
+        .catch(function(response) {
+          var msg = interpolate(gettext('Unable to delete portgroup: %s'),
+                                [response.data],
+                                false);
+          toastService.add('error', msg);
+          return $q.reject(msg);
+        });
+    }
+
+    /**
+     * @description Update the definition of a specified portgroup.
+     *
+     * http://developer.openstack.org/api-ref/baremetal/#update-a-portgroup
+     *
+     * @param {string} portgroupId – UUID or name of a portgroup.
+     * @param {object[]} patch – Sequence of update operations
+     * @return {promise} Promise
+     */
+    function updatePortgroup(portgroupId, patch) {
+      return apiService.patch('/api/ironic/portgroups/' + portgroupId,
+                              {patch: patch})
+        .then(function(response) {
+          var msg = gettext('Successfully updated portgroup %s');
+          toastService.add('success', interpolate(msg, [portgroupId], false));
+          return response.data; // The updated portgroup
+        })
+        .catch(function(response) {
+          var msg = interpolate(gettext('Unable to update portgroup %s: %s'),
+                                [portgroupId, response.data],
+                                false);
+          toastService.add('error', msg);
+          return $q.reject(msg);
+        });
+    }
+
+    /**
+     * @description Get the ports associated with a specified portgroup.
+     *
+     * http://developer.openstack.org/api-ref/baremetal/#list-ports-by-portgroup
+     *
+     * @param {string} portgroupId – UUID or name of the portgroup.
+     * @return {promise} Promise containing a list of ports.
+     */
+    function getPortgroupPorts(portgroupId) {
+      return apiService.get(
+        '/api/ironic/portgroups/' + portgroupId + '/ports')
+        .then(function(response) {
+          return response.data.ports; // List of ports
+        })
+        .catch(function(response) {
+          var msg = interpolate(
+            gettext('Unable to retrieve portgroup ports: %s'),
+            [response.data],
+            false);
+          toastService.add('error', msg);
+          return $q.reject(msg);
+        });
+    }
+  }
 }());
